/*
    Copyright (C) 2006 Carlos Olmedo Escobar <arroba2puntos@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "cpu.h"

int main()	// Just for testing
{
    Processor cpu;
    int i;
    QMap<QString, float> map_of_percetages;
    
    for ( i = 0; i < 5; i++ )
        {
        cpu.getUsage( &map_of_percetages );
        std::cout << "idle: " << map_of_percetages.value( "idle" ) << " %\n";
        sleep( 1 );
        }

    for ( i = 0; i < 5; i++ )
        {
      	cpu.getUsage( &map_of_percetages, 0 );
      	std::cout << "user: " << map_of_percetages.value( "user" ) << " %\n";
        sleep( 1 );
        }

}



Processor::Processor()
{
    length = sizeof(cpu_time);
}



Processor::~Processor()
{
}



bool Processor::getUsage( QMap<QString, float> * map_to_fill )
{
    if ( sysctlbyname("kern.cp_time", &cpu_time, &length, NULL, 0) )
        return false;
    percentages(CPUSTATES, cpu_states, cpu_time, cpu_old, cpu_diff);

    map_to_fill->insert( "user", (float) cpu_states[CP_USER] / 10.0 );
    map_to_fill->insert( "nice", (float) cpu_states[CP_NICE] / 10.0 );
    map_to_fill->insert( "system", (float) cpu_states[CP_SYS] / 10.0 );
    map_to_fill->insert( "idle", (float) cpu_states[CP_IDLE] / 10.0 );
    map_to_fill->insert( "interrupt", (float) cpu_states[CP_INTR] / 10.0 );
    
    return true;
}



/* This method is ripped from top.
 */
/*
 *  Top users/processes display for Unix
 *  Version 3
 *
 *  This program may be freely redistributed,
 *  but this entire comment MUST remain intact.
 *
 *  Copyright (c) 1984, 1989, William LeFebvre, Rice University
 *  Copyright (c) 1989, 1990, 1992, William LeFebvre, Northwestern University
 */
void Processor::percentages( int cnt, int * out, long *newe, long * old, long * diffs )
{
    register short i;
    register long change;
    register long total_change;
    register long * dp;
    long half_total;

    /* initialization */
    total_change = 0;
    dp = diffs;

    /* calculate changes for each state and the overall change */
    for ( i = 0; i < cnt; i++ )
        {
        if ( ( change = *newe - *old ) < 0 )
            {
            /* this only happens when the counter wraps */
            change = ( int ) ( ( unsigned long ) * newe - ( unsigned long ) * old );
            }
        total_change += ( *dp++ = change );
        *old++ = *newe++;
        }

    /* avoid divide by zero potential */
    if ( total_change == 0 )
        total_change = 1;

    /* calculate percentages based on overall change, rounding up */
    half_total = total_change / 2l;
    for ( i = 0; i < cnt; i++ )
        {
        *out++ = ( int ) ( ( *diffs++ * 1000 + half_total ) / total_change );
        }
}
