/*
    Copyright (C) 2006 Carlos Olmedo Escobar <arroba2puntos@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef CPU_H
#define CPU_H

#include <QMap>
#include <QString>

#include <sys/sysctl.h>
#include <osreldate.h>
#if __FreeBSD_version < 500101
	#include <sys/dkstat.h>
#else
	#include <sys/resource.h>
#endif
#include <iostream>


/**
 * This class retrieves information about the cpu usage in FreeBSD.
 *
 * @author Carlos Olmedo Escobar <arroba2puntos@gmail.com>
 */
class Processor
{

public:
    /**
     * Constructs a processor object.
     */
    Processor();
    
    /**
     * Destroys a processor object.
     */
    ~Processor();
    
    /**
     * Retrieves the diferent usage percentages from the overall cpus
     * @param map_to_fill The associative container where the data is 
     * stored. Key values refers to types of percentages.
     * Key values: "user", "nice", "system" and "idle".
     * Non transportable values: "interrupt".
     * @return false if occurs an error, true otherwise.
     */
    bool getUsage( QMap<QString, float> * map_to_fill );

private:
    long cpu_time[CPUSTATES];
    long cpu_old[CPUSTATES];
    long cpu_diff[CPUSTATES];
    int cpu_states[CPUSTATES];
    size_t length;
    
    /**
     * Calculates the percentages
     * @param cnt the number of percentages
     * @param out an array with the calculated percentages
     * @param newe an array of values obtained from sysctlbyname
     * @param old and array of values obtained from a previous call
     * @param diff an array with the difference between the old and 
     * newe values
     */
    void percentages( int cnt, int * out, long * newe, long * old, long * diffs );
    
};

#endif
